##                                                                 -*- Perl -*-
##  2000-2001 Patryk Zadarnowski <pat@jantar.org>
#
# Redistribution  and  use  in  source  and  binary  forms,  with  or  without
# modification, are permitted provided  that the following conditions are met:
# (1) Redistributions of  source code must retain the  above copyright notice,
# this list  of conditions and the following  disclaimer.  (2) Redistributions
# in  binary form  must reproduce  the above  copyright notice,  this  list of
# conditions and  the following disclaimer  in the documentation  and/or other
# materials provided  with the distribution.  (3)  The name of  any author may
# not  be used  to  endorse or  promote  products derived  from this  software
# without their specific prior written permission.
#
# THIS SOFTWARE IS  PROVIDED ``AS IS'' AND ANY  EXPRESS OR IMPLIED WARRANTIES,
# INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
# FITNESS FOR  A PARTICULAR  PURPOSE, ARE DISCLAIMED.   IN NO EVENT  SHALL THE
# AUTHORS BE LIABLE FOR  ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
# OR  CONSEQUENTIAL DAMAGES  (INCLUDING, BUT  NOT LIMITED  TO,  PROCUREMENT OF
# SUBSTITUTE  GOODS OR SERVICES;  LOSS OF  USE, DATA  OR PROFITS;  OR BUSINESS
# INTERRUPTION)  HOWEVER CAUSED  AND ON  ANY THEORY  OF LIABILITY,  WHETHER IN
# CONTRACT,  STRICT LIABILITY,  OR  TORT (INCLUDING  NEGLIGENCE OR  OTHERWISE)
# ARISING IN ANY WAY  OUT OF THE USE OF THIS SOFTWARE,  EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

# This perl script generates three variables that encode the Unicode version
# in the used by the library:
#
#	const int __unidata_version_major;
#	const int __unidata_version_minor;
#	const int __unidata_version_update;
#
# The data is extracted from the first line of "PropList.txt", as there seems
# to be no better place to get it from.

my $progress_limit;
my $progress_current;
my $progress_running = 0;

sub begin_progress_bar {
    $progress_current = "  0";
    $progress_running = 1;
    $progress_limit = shift;
    print STDERR "@_: $progress_current\%";
}

sub update_progress_bar {
    my $x = sprintf "\%3.0f", ($_[0] / $progress_limit * 100);
    if ($x ne $progress_current) {
	printf STDERR "\b\b\b\b$x\%";
	$progress_current = $x;
    }
}

sub end_progress_bar {
    print STDERR "\b\b\b\b100\%\n";
    $progress_running = 0;
}

sub panic {
    my $header = shift;
    print STDERR "\n" if $progress_running;
    print STDERR "[PropList.pl] $header \n";
    foreach $line (@_) {
	print STDERR "\t$line\n";
    }
    exit 1;
}

sub addslashes {
    $_[0] =~ s/[\"\\]/\\\&/g;
    return $_[0];
}

# Read and parse the first line of the "PropList.txt" file:

my $src = "PropList";

begin_progress_bar 1, "Loading \"$src\"";
open IN, "< $src.txt" or panic "Cannot open \"$src.txt\":", $!;

$line = <IN>;
$line =~ m/Property\s+dump:\s+UnicodeData-([0-9]+)\.([0-9]+)\.([0-9]+)\.txt/i
    or panic "Unknown or corrupted \"$src.txt\".";
close(IN);
end_progress_bar;

# Generate "version.c":
print STDERR "Generating \"$src.c\":\n";
open OUT, "> $src.c" or panic "Cannot create \"$src.c\":", $!;
print OUT "/* This file has been automatically generated. Do not edit! */

#include \"unidata.h\"

";

begin_progress_bar 1, ' Generating "__unidata_version_major"';
print OUT "const int __unidata_version_major  = $1;\n";
end_progress_bar;

begin_progress_bar 1, ' Generating "__unidata_version_minor"';
print OUT "const int __unidata_version_minor  = $2;\n";
end_progress_bar;

begin_progress_bar 1, ' Generating "__unidata_version_update"';
print OUT "const int __unidata_version_update  = $3;\n";
end_progress_bar;

close(OUT);
